@@ -1,6 +1,6 @@
 /***************************************************************************
  *   2007-2016 by Peter Semiletov                                          *
- *   tea@list.ru                                                           *
+ *   peter.semiletov@gmail.com                                                           *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
  *   it under the terms of the GNU General Public License as published by  *
@@ -46,7 +46,8 @@ code from qwriter:
 #include "textproc.h"
 
 
-
+#include <QApplication>
+#include <QClipboard>
 #include <QSettings>
 #include <QLabel>
 #include <QPainter>
@@ -70,6 +71,8 @@ QHash <QString, QString> global_palette;
 QSettings *settings;
 QMenu *current_files_menu;
 
+int recent_list_max_items;
+
 bool b_recent_off;
 bool b_destroying_all;
 
@@ -400,6 +403,7 @@ CDocument* document_holder::open_file (const QString &fileName, const QString &c
   if (d)
      {
       tab_widget->setCurrentIndex (tab_widget->indexOf (d->tab_page));
+      d->reload (codec);
       return d;
      }
 
@@ -642,6 +646,9 @@ void document_holder::apply_settings_single (CDocument *d)
 
   int darker_val = settings->value ("darker_val", 100).toInt();
 
+ 
+
+
 /*  
   f.fromString (settings->value ("editor_font_name", "Monospace").toString());
   f.setPointSize (settings->value ("editor_font_size", "12").toInt());
@@ -657,6 +664,15 @@ void document_holder::apply_settings_single (CDocument *d)
   
   d->textEdit->wrap = settings->value ("word_wrap", true).toBool();
     
+     
+  QString s_sel_back_color = hash_get_val (global_palette, "sel-background", "black");
+  QString s_sel_text_color = hash_get_val (global_palette, "sel-text", "white");
+
+    
+  d->textEdit->sel_text_color = QColor (s_sel_text_color).darker(darker_val).name(); 
+  d->textEdit->sel_back_color = QColor (s_sel_back_color).darker(darker_val).name(); 
+
+    
   if (! d->textEdit->use_hl_wrap)
      {
       if (d->textEdit->wrap)
@@ -743,7 +759,8 @@ void document_holder::add_to_recent (CDocument *d)
   s += QString ("%1").arg (d->textEdit->textCursor().position());
 
   recent_files.prepend (s);
-  if (recent_files.size() > 13)
+//  if (recent_files.size() > settings->value ("recent_list.max_items", 21).toInt())
+  if (recent_files.size() > recent_list_max_items)
      recent_files.removeLast();
 }
 
@@ -833,7 +850,8 @@ void CTEAEdit::cb_cursorPositionChanged()
      doc->update_status();
      
   if (hl_brackets)   
-     braceHighlight();   
+     //braceHighlight();   
+     update_ext_selections();
 }
 
 
@@ -932,11 +950,14 @@ void CTEAEdit::setup_brace_width()
 CTEAEdit::CTEAEdit (QWidget *parent): QPlainTextEdit (parent)
 {
   rect_sel_reset();
-
+  
   highlightCurrentLine = false;
   setup_brace_width();
   
   lineNumberArea = new CLineNumberArea (this);
+
+  connect(this, SIGNAL(selectionChanged()), this, SLOT(slot_selectionChanged()));
+  
   connect(this, SIGNAL(blockCountChanged(int)), this, SLOT(updateLineNumberAreaWidth(int)));
   connect(this, SIGNAL(updateRequest(const QRect &, int)), this, SLOT(updateLineNumberArea(const QRect &, int)));
   updateLineNumberAreaWidth (0);
@@ -954,6 +975,16 @@ CTEAEdit::CTEAEdit (QWidget *parent): QPlainTextEdit (parent)
   //document()->setDefaultTextOption(QTextOption(Qt::AlignRight));
  // setCursorWidth (settings->value ("cursor_width", 1).toInt());
   
+  
+  QString s_sel_back_color = hash_get_val (global_palette, "sel-background", "black");
+  QString s_sel_text_color = hash_get_val (global_palette, "sel-text", "white");
+
+  int darker_val = settings->value ("darker_val", 100).toInt();
+
+  sel_text_color = QColor (s_sel_text_color).darker(darker_val).name(); 
+  sel_back_color = QColor (s_sel_back_color).darker(darker_val).name(); 
+  
+  
   connect (this, SIGNAL(cursorPositionChanged()), this, SLOT(cb_cursorPositionChanged()));
 }
 
@@ -1953,9 +1984,10 @@ void CTEAEdit::resizeEvent (QResizeEvent *e)
 
 void CTEAEdit::braceHighlight()
 {
-  extraSelections.clear();
-  setExtraSelections (extraSelections);
-  selection.format.setBackground (brackets_color);
+  //extraSelections.clear();
+  //setExtraSelections (extraSelections);
+  
+  brace_selection.format.setBackground (brackets_color);
 
   QTextDocument *doc = document();
   QTextCursor cursor = textCursor();
@@ -2026,10 +2058,10 @@ void CTEAEdit::braceHighlight()
       QTextCursor cursor2 = doc->find (openBrace, cursor);
       if (cursor2.isNull()) 
          {
-          selection.cursor = cursor;
-          extraSelections.append (selection);
-          selection.cursor = cursor1;
-          extraSelections.append (selection);
+          brace_selection.cursor = cursor;
+          extraSelections.append (brace_selection);
+          brace_selection.cursor = cursor1;
+          extraSelections.append (brace_selection);
           setExtraSelections (extraSelections);
          } 
       else 
@@ -2042,10 +2074,10 @@ void CTEAEdit::braceHighlight()
                       break;
                  }
                 
-           selection.cursor = cursor;
-           extraSelections.append (selection);
-           selection.cursor = cursor1;
-           extraSelections.append (selection);
+           brace_selection.cursor = cursor;
+           extraSelections.append (brace_selection);
+           brace_selection.cursor = cursor1;
+           extraSelections.append (brace_selection);
            setExtraSelections (extraSelections);
           }
        } 
@@ -2057,10 +2089,10 @@ void CTEAEdit::braceHighlight()
             QTextCursor cursor2 = doc->find (closeBrace, cursor, QTextDocument::FindBackward);
             if (cursor2.isNull()) 
                {
-                selection.cursor = cursor;
-                extraSelections.append (selection);
-                selection.cursor = cursor1;
-                extraSelections.append (selection);
+                brace_selection.cursor = cursor;
+                extraSelections.append (brace_selection);
+                brace_selection.cursor = cursor1;
+                extraSelections.append (brace_selection);
                 setExtraSelections (extraSelections);
                }
             else 
@@ -2073,10 +2105,10 @@ void CTEAEdit::braceHighlight()
                            break;
                        }
                        
-                 selection.cursor = cursor;
-                 extraSelections.append (selection);
-                 selection.cursor = cursor1;
-                 extraSelections.append (selection);
+                 brace_selection.cursor = cursor;
+                 extraSelections.append (brace_selection);
+                 brace_selection.cursor = cursor1;
+                 extraSelections.append (brace_selection);
                  setExtraSelections (extraSelections);
                 }
              }
@@ -2084,18 +2116,26 @@ void CTEAEdit::braceHighlight()
 }
 
 
+bool CTEAEdit::has_rect_selection()
+{
+ if (rect_sel_start.y() == -1 || rect_sel_end.y() == -1)
+     return false; 
+     
+ return true;    
+}
+ 
+
 void CTEAEdit::update_rect_sel()
 {
-  qDebug() << "CTEAEdit::update_rect_sel()  -1";
+ // qDebug() << "CTEAEdit::update_rect_sel()  -1";
  
   if (rect_sel_start.y() == -1 || rect_sel_end.y() == -1)
      return; 
  
   QTextEdit::ExtraSelection rect_selection;
 
-  extraSelections.clear();
-  
-  
+  //extraSelections.clear();
+    
   int y1 = std::min (rect_sel_start.y(), rect_sel_end.y());
   int y2 = std::max (rect_sel_start.y(), rect_sel_end.y());
   int ydiff = y2 - y1;
@@ -2105,6 +2145,7 @@ void CTEAEdit::update_rect_sel()
   int x2 = std::max (rect_sel_start.x(), rect_sel_end.x());
   int xdiff = x2 - x1;
  
+  int correction = 0;
 
   QTextCursor cursor = textCursor();
   
@@ -2114,22 +2155,41 @@ void CTEAEdit::update_rect_sel()
   for (int y = y1; y <= y2; y++)
       {
 //       qDebug() << "y:" << y;
-       cursor.movePosition (QTextCursor::Right, QTextCursor::MoveAnchor, x1);
-              
-       int sel_len = xdiff;
        
        QTextBlock b = document()->findBlockByNumber (y); 
        
+   //    qDebug() << "b.text().length(): " << b.text().length();
+     //  qDebug() << "x1: " << x1;
+       
+       if (b.text().length() == 0)
+          {
+           //cursor.movePosition (QTextCursor::NextBlock, QTextCursor::MoveAnchor);
+           correction++;
+           continue;
+          } 
+       
+       int sel_len = xdiff;
+       
        if ((b.text().length() - x1) < xdiff)
           sel_len = b.text().length() - x1;
-          
+   
+       cursor.movePosition (QTextCursor::Right, QTextCursor::MoveAnchor, x1 + correction);
+
        cursor.movePosition (QTextCursor::Right, QTextCursor::KeepAnchor, sel_len);
               
        rect_selection.cursor = cursor;
-       rect_selection.format.setBackground (brackets_color);
-       extraSelections.append (rect_selection);
+      // rect_selection.format.setBackground (brackets_color);
+      rect_selection.format.setBackground (sel_back_color);
+      rect_selection.format.setForeground (sel_text_color);
+      
+      
+      extraSelections.append (rect_selection);
   
-       cursor.movePosition (QTextCursor::NextBlock, QTextCursor::MoveAnchor);
+      cursor.movePosition (QTextCursor::NextBlock, QTextCursor::MoveAnchor);
+       
+      if (b.text().length() != 0)
+          correction = 0;
+       
       }
 
   setExtraSelections (extraSelections);
@@ -2140,12 +2200,13 @@ void CTEAEdit::update_rect_sel()
   qDebug() << "x1:" << x1;
   qDebug() << "x2:" << x2;
   */
-  qDebug() << get_rect_sel();
+ // qDebug() << get_rect_sel();
   
 }
+
+
   
-  
-void CTEAEdit::rect_sel_replace (const QString &s)
+void CTEAEdit::rect_sel_replace (const QString &s, bool insert)
 {
 /*
 
@@ -2157,8 +2218,8 @@ void CTEAEdit::rect_sel_replace (const QString &s)
 
 */
 
-  int y = textCursor().block().blockNumber();
-  int x = textCursor().position() - textCursor().block().position();
+ // int y = textCursor().block().blockNumber();
+ // int x = textCursor().position() - textCursor().block().position();
  
   int y1 = std::min (rect_sel_start.y(), rect_sel_end.y());
   int y2 = std::max (rect_sel_start.y(), rect_sel_end.y());
@@ -2171,30 +2232,133 @@ void CTEAEdit::rect_sel_replace (const QString &s)
   
   int how_many_copy_from_source = ydiff;
   
-  int lines_to_end = blockCount() - y;
+  int lines_to_end = blockCount() - y1;
   
   if (ydiff > lines_to_end)
      how_many_copy_from_source = lines_to_end;
      
   QStringList sl_source;
   
-  
-  for (int line = y1; y <= y2; y++)
+  for (int line = y1; line <= y2; line++)
       {
-       //int sel_len = xdiff;
-       
        QTextBlock b = document()->findBlockByNumber (line); 
-       
-       //if ((b.text().length() - x1) < xdiff)
-         // sel_len = b.text().length() - x1;
-          
        sl_source.append (b.text());   
-          
       }  
   
+  QStringList sl_insert = s.split ("\n");
   
+  QStringList sl_dest;
+  
+  for (int line = 0; line < sl_insert.size(); line++)
+      { 
+       QString t;
+       
+       if (line >= sl_source.size())
+          {
+           t = sl_insert [line];
+           sl_dest.append (t);
+           continue;  
+          }
+  
+       t = sl_source[line].left (x1);
+       t += sl_insert [line];
+       
+       if (! insert)
+          t += sl_source[line].mid (x2);
+       else   
+           t += sl_source[line].mid (x1);
+  
+       sl_dest.append (t);
+      }
+
+  QString new_text = sl_dest.join ('\n');
+  //new_text += "\n";
+
+//теперь выделить при помощи курсора всё от y1 до y2 и обычным способом заменить текст     
+  
+  QTextCursor cursor = textCursor();
+  
+  cursor.movePosition (QTextCursor::Start, QTextCursor::MoveAnchor);
+  cursor.movePosition (QTextCursor::NextBlock, QTextCursor::MoveAnchor, y1);
+  cursor.movePosition (QTextCursor::NextBlock, QTextCursor::KeepAnchor, ydiff);
+  cursor.movePosition (QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+  
+  cursor.removeSelectedText();
+  
+  textCursor().insertText (new_text);
+  
+//  qDebug() << new_text;
 }
-    
+  
+/*  
+void CTEAEdit::rect_sel_insert (const QString &s)
+{
+ 
+  int y1 = std::min (rect_sel_start.y(), rect_sel_end.y());
+  int y2 = std::max (rect_sel_start.y(), rect_sel_end.y());
+  int ydiff = y2 - y1;
+   
+  
+  int x1 = std::min (rect_sel_start.x(), rect_sel_end.x());
+  int x2 = std::max (rect_sel_start.x(), rect_sel_end.x());
+  int xdiff = x2 - x1;
+  
+  int how_many_copy_from_source = ydiff;
+  
+  int lines_to_end = blockCount() - y1;
+  
+  if (ydiff > lines_to_end)
+     how_many_copy_from_source = lines_to_end;
+     
+  QStringList sl_source;
+  
+  for (int line = y1; line <= y2; line++)
+      {
+       QTextBlock b = document()->findBlockByNumber (line); 
+       sl_source.append (b.text());   
+      }  
+  
+  QStringList sl_insert = s.split ("\n");
+  
+  QStringList sl_dest;
+  
+  for (int line = 0; line < sl_insert.size(); line++)
+      { 
+       QString t;
+       
+       if (line >= sl_source.size())
+          {
+           t = sl_insert [line];
+           sl_dest.append (t);
+           continue;  
+          }
+  
+       t = sl_source[line].left (x1);
+       t += sl_insert [line];
+       t += sl_source[line].mid (x1);
+  
+       sl_dest.append (t);
+      }
+
+  QString new_text = sl_dest.join ('\n');
+  //new_text += "\n";
+
+//теперь выделить при помощи курсора всё от y1 до y2 и обычным способом заменить текст     
+  
+  QTextCursor cursor = textCursor();
+  
+  cursor.movePosition (QTextCursor::Start, QTextCursor::MoveAnchor);
+  cursor.movePosition (QTextCursor::NextBlock, QTextCursor::MoveAnchor, y1);
+  cursor.movePosition (QTextCursor::NextBlock, QTextCursor::KeepAnchor, ydiff);
+  cursor.movePosition (QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+  
+  cursor.removeSelectedText();
+  
+  textCursor().insertText (new_text);
+  
+  qDebug() << new_text;
+}
+ */   
 
 bool CTEAEdit::canInsertFromMimeData (const QMimeData *source)
 {
@@ -2205,6 +2369,20 @@ bool CTEAEdit::canInsertFromMimeData (const QMimeData *source)
 }
 
 
+QMimeData* CTEAEdit::createMimeDataFromSelection()
+{
+  qDebug() << "CTEAEdit::createMimeDataFromSelection()";
+  if (has_rect_selection())
+     {
+      QMimeData *md = new QMimeData;
+      md->setText (get_rect_sel());
+      return md;
+     }
+
+  return QPlainTextEdit::createMimeDataFromSelection();   
+} 
+
+
 void CTEAEdit::insertFromMimeData (const QMimeData *source)
 {
   QString fName;
@@ -2270,34 +2448,11 @@ void document_holder::open_current()
 {
   QAction *act = qobject_cast<QAction *>(sender());
 
-  //QStringList current_files; 
- 
-/* foreach (CDocument *d, list)
-         current_files.prepend (d->file_name);
-  
-  int i = current_files.indexOf (act->text());
-  if (i == -1)
-     return;
-*/
- // open_file_triplex (current_files[i]);
- 
   CDocument *d = get_document_by_fname (act->text());
   if (d)
       tab_widget->setCurrentIndex (tab_widget->indexOf (d->tab_page));
- 
- 
-//  update_current_files_menu();
 }
 
-/*
-void document_holder::add_to_current_files (CDocument *d)
-{
-  
-  recent_files.prepend (d->file_name);
-  //if (recent_files.size() > 13)
-     //recent_files.removeLast();
-}
-*/
 
 void document_holder::update_current_files_menu()
 {
@@ -2355,6 +2510,114 @@ QString CTEAEdit::get_rect_sel()
 }
 
 
+void CTEAEdit::rect_sel_cut (bool just_del)
+{
+  int y1 = std::min (rect_sel_start.y(), rect_sel_end.y());
+  int y2 = std::max (rect_sel_start.y(), rect_sel_end.y());
+  int ydiff = y2 - y1;
+   
+  
+  int x1 = std::min (rect_sel_start.x(), rect_sel_end.x());
+  int x2 = std::max (rect_sel_start.x(), rect_sel_end.x());
+  int xdiff = x2 - x1;
+  
+  int how_many_copy_from_source = ydiff;
+  
+  int lines_to_end = blockCount() - y1;
+  
+  if (ydiff > lines_to_end)
+     how_many_copy_from_source = lines_to_end;
+     
+  QStringList sl_source;
+  
+  for (int line = y1; line <= y2; line++)
+      {
+       QTextBlock b = document()->findBlockByNumber (line); 
+       sl_source.append (b.text());   
+      }  
+  
+  QStringList sl_dest;
+  QStringList sl_copy;
+  
+  for (int line = 0; line < sl_source.size(); line++)
+      { 
+       QString t;
+
+       t = sl_source[line].left (x1);
+       t += sl_source[line].mid (x2);
+       sl_dest.append (t);
+       
+       sl_copy.append (sl_source[line].mid (x1, x2 - x1));
+      }
+
+  QString new_text = sl_dest.join ('\n');
+
+//теперь выделить при помощи курсора всё от y1 до y2 и обычным способом заменить текст     
+  
+  QTextCursor cursor = textCursor();
+  
+  cursor.movePosition (QTextCursor::Start, QTextCursor::MoveAnchor);
+  cursor.movePosition (QTextCursor::NextBlock, QTextCursor::MoveAnchor, y1);
+  cursor.movePosition (QTextCursor::NextBlock, QTextCursor::KeepAnchor, ydiff);
+  cursor.movePosition (QTextCursor::EndOfBlock, QTextCursor::KeepAnchor);
+  
+  cursor.beginEditBlock();
+  
+  cursor.removeSelectedText();
+ 
+  cursor.insertText (new_text);
+
+  cursor.endEditBlock();
+  
+  if (! just_del)  
+     QApplication::clipboard()->setText (sl_copy.join ('\n'));
+}
+
+
+void CTEAEdit::update_ext_selections()
+{
+  extraSelections.clear();
+  setExtraSelections (extraSelections);
+  update_rect_sel();
+  braceHighlight();     
+}
+  
+
+void CTEAEdit::slot_selectionChanged()
+{
+  QTextCursor cursor = this->textCursor();
+  
+  if (cursor.selectionStart() != cursor.selectionEnd())
+     {
+      rect_sel_start.setX (-1);
+      rect_sel_end.setX (-1);
+      update_ext_selections();
+     } 
+}
+
+
+void CTEAEdit::text_replace (const QString &s)
+{
+  if (has_rect_selection())
+     rect_sel_replace (s, false);
+  else
+      textCursor().insertText (s);
+}
+
+
+
+/*
+void CTEAEdit::copy()
+{
+  qDebug() << "CTEAEdit::copy()";
+  
+  if (has_rect_selection())
+     QApplication::clipboard()->setText (get_rect_sel());   
+  else   
+      QPlainTextEdit::copy();
+}
+*/
+
 /*
  * 
  
