@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyleft 2007-2014 by Peter Semiletov <tea@list.ru>                   *
+ *   Copyleft 2007-2017 by Peter Semiletov <peter.semiletov@gmail.com>     *
  *                                                                         *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -26,6 +26,9 @@
 
 #include "single_application.h"
 
+#ifndef Q_OS_OS2
+#include "single_application_shared.h"
+#endif
 
 extern rvln *mainWindow; 
 
@@ -36,23 +39,21 @@ int main (int argc, char *argv[])
    
 #if defined(Q_OS_OS2)
 
-  CApplication app (argc, argv);
+  QApplication app (argc, argv);
   qApp->setApplicationName ("TEA");
-  
-  //rvln mw;
-
-  //mw.show();
-  
+    
   mainWindow = new rvln();
   mainWindow->show();
 
   return app.exec();
   
-#else
+#endif
   
   //Q_INIT_RESOURCE (rlvn);
+
+#if defined(Q_OS_WIN)
   
-  SingleApplication app (argc, argv, "tea unique id 1977");
+  CSingleApplication app (argc, argv, "tea unique id 1977");
   
   if (app.isRunning())
      {
@@ -60,19 +61,41 @@ int main (int argc, char *argv[])
        if (argc > 1)
           for (int i = 1; i < argc; i++) 
                app.sendMessage (QString(argv[i]));
-       
        return 0;
      }
 
+#endif
 
+
+#if defined(Q_OS_UNIX)
+
+ CSingleApplicationShared app (argc, argv, "tea unique id 1977");
+ 
+ if (app.alreadyExists())
+    {
+     if (argc > 1)
+        for (int i = 1; i < argc; i++) 
+             app.sendMessage (QString(argv[i]));
+             
+     return 0;
+    }
+    
+#endif     
+     
   mainWindow = new rvln();
 
-  QObject::connect(&app, SIGNAL(messageAvailable(QString)), mainWindow, SLOT(receiveMessage(QString)));
-  
-  mainWindow->show();
+#if defined(Q_OS_WIN)
+ QObject::connect(&app, SIGNAL(messageAvailable(QString)), mainWindow, SLOT(receiveMessage(QString)));
+#endif
 
+#if defined(Q_OS_UNIX)
+ QObject::connect(&app, SIGNAL(messageAvailable(QStringList)), mainWindow, SLOT(receiveMessageShared(QStringList)));
+// QObject::connect(&app, SIGNAL(signal_commit_data()), mainWindow, SLOT(app_commit_data()));
+// QObject::connect(&app, SIGNAL(commitDataRequest(QSessionManager & )), mainWindow, SLOT(slot_commitDataRequest(QSessionManager & )));
+
+#endif
+   
+  mainWindow->show();
   
   return app.exec();   
-  
-#endif  
 }
