@@ -1,5 +1,5 @@
 /**************************************************************************
- *   Copyright (C) 2007-2012 by Peter Semiletov                            *
+ *   2007-2017 by Peter Semiletov                            *
  *   peter.semiletov@gmail.com                                             *
  *                                                                         *
  *   This program is free software; you can redistribute it and/or modify  *
@@ -51,7 +51,7 @@
 #include <QTextBrowser>
 #include <QTranslator>
 #include <QProcess>
-
+#include <QSessionManager>
 //#include <QSystemTrayIcon>
 
 
@@ -75,19 +75,6 @@
 #endif
 
 
-/*
-class CMarkupPair: public QObject
-{
-  Q_OBJECT
-
-public:
-
-  QHash <QString, QString> tags_a;
-  QHash <QString, QString> tags_b;
-};
-*/
-
-
 
 class CMarkupPair: public QObject
 {
@@ -389,10 +376,16 @@ public slots:
 main window callbacks
 *************************/
 
+  void app_commit_data();
+
+  void slot_commitDataRequest(QSessionManager &m); 
+
   void repeat();
 
   void receiveMessage(const QString &msg);
   
+  void receiveMessageShared(const QStringList& msg);
+        
   void view_use_theme();
 
 
@@ -414,6 +407,12 @@ main window callbacks
   void fman_refresh();
   void fman_rename();
   void fman_delete();
+  void fman_zeropad();
+  void fman_del_n_first_chars();
+  void fman_multreplace();
+  void fman_apply_template();
+
+  
   void fman_drives_changed (const QString & path);
   void fman_current_file_changed (const QString &full_path, const QString &just_name);
   void fman_file_activated (const QString &full_path);
@@ -501,6 +500,9 @@ main window callbacks
 
   void ed_block_start();
   void ed_block_end();
+  void ed_block_copy();
+  void ed_block_paste();
+  void ed_block_cut();
   
 
   void ed_copy();
@@ -563,6 +565,11 @@ main window callbacks
   void cb_script_finished (int exitCode, QProcess::ExitStatus exitStatus);
   void fn_number_arabic_to_roman();
   void fn_number_roman_to_arabic();
+  
+  void fn_number_dms2dc();
+  void fn_number_dd2dms();
+
+  
   void fn_filter_rm_less_than();
   void fn_filter_rm_greater_than();
   void fn_filter_rm_duplicates();
@@ -850,6 +857,8 @@ main window widgets
 
   QMenu *menu_fm;
   QMenu *menu_fm_file_ops;
+  QMenu *menu_fm_multi_rename;
+
   QMenu *menu_fm_file_infos;
   QMenu *menu_fm_img_conv;
   QMenu *menu_fm_zip;
@@ -1094,19 +1103,6 @@ prefs window widgets
 };
 
 
-class CApplication: public QApplication
-{
-  Q_OBJECT
-
-public:
-
-  CApplication (int &argc, char **argv): QApplication (argc, argv)
-               {;}
-
-  void saveState (QSessionManager &manager);
-};
-
-
 #if QT_VERSION >= 0x050000
 
 class CQQuickWindow: public QQuickWindow
@@ -1123,7 +1119,6 @@ public:
 
 protected:
 
-//  void closeEvent (QCloseEvent *event);
   bool event (QEvent *event);
 
   
