@@ -316,10 +316,30 @@ bool char_is_shit (const QChar &c)
   
   if (c == 0xfeff00bb)
        return false;*/
-
   
+  /*if (c.isNull())
+     return false;
+
+
+  if (c == '\"')
+     return true;
+
+  //if (c.unicode() == 187) //'»')
+    // return true;
+*/
+ // if (c == '\\')
+   //  return true;
+  
+  //if (! c.isLetter())
+    // return true;
+ 
  if (! c.isNull() && ! c.isLetter())
      return true;
+ 
+ //if (! c.isNull() && c.unicode() == 187)
+   //  return true;
+   
+  
   
   return false;
 }
